#region Using declarations
using System;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// DoubleMAPaint, originally by Rollins on NinjaTrader Forums
	/// Modified to add additional moving average types by Big Mike (ctrlbrk.blogspot.com) 05/06/2009
	/// Modified to add Signal dataseries (-1 short, 0 neutral, 1 long) by ctrlbrk
	/// Modified to add PaintBars by sam028 2009/06/08
	/// Modified to add sound alerts by sam028 2009/06/11
	/// Modified to add different .wav file alerts by sam028 2009/06/12
	/// I also added DMV dataseries to easily access correct closing value for each bar unlike original design 05/18/2009
    /// </summary>
    [Description("DoubleMAPaint")]
    [Gui.Design.DisplayName("Double MA Paint")]
    public class DoubleMAPaint : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int mA2Period = 34; // Default setting for MAPeriod
            private int mA1Period = 34; // Default setting for MAPeriod
			private NinjaTrader.Indicator.MAV.MAType mA1Type = NinjaTrader.Indicator.MAV.MAType.EMA;
			private NinjaTrader.Indicator.MAV.MAType mA2Type = NinjaTrader.Indicator.MAV.MAType.EMA;
			private Color risingColor;
			private Color fallingColor;
			private bool showArrows = false;
			private int arrowDisplacement = 7;
		// User defined variables (add any user defined variables below)
			private int direction;
			private int lastDirection;
			private int lastCalcBar = -2;
			private bool drawObjects = false;
			private ArrayList data = new ArrayList();

			private DataSeries signal;
			private DataSeries dmv;
			private int decimals;
		// Paint bars
			private bool paintBar = true;
			private Color barColorUp = Color.Blue;
			private Color barColorDown = Color.Red;
			private Color barColorNeutral = Color.Yellow;
		// Sound	
			private bool soundon=false;
			private string longwavfilename = "long.wav";
			private string shortwavfilename="short.wav";
			
		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, " RisingPlot") );
            Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, " FallingPlot") );
            Add(new Plot(Color.FromKnownColor(KnownColor.Yellow), PlotStyle.Line, " NeutralPlot"));
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= true;
			
			Plots[0].Pen.Width = 4;
			Plots[1].Pen.Width = 4;
			Plots[2].Pen.Width = 4;
			
			dmv = new DataSeries(this);
			signal = new DataSeries(this);
			//triEma = EMA( EMA( EMA( this, Period ), Period ), Period );
			lastDirection = -2;
			double double1 = TickSize;
			int max = 30;
			int i = 0;
			while (double1 % 1 !=0 )
			{
				double1 *= 10;
				decimals++;
				i++;
				if( i > max )
					break;
			}
			if( decimals > 0 )
				decimals++;
			//Print( TickSize + " dec " + decimals );
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < Math.Max(MA1Period * 2, MA2Period * 2)) return;
			
			try
			{
				if( lastCalcBar != CurrentBar )
				{
					lastCalcBar = CurrentBar;
					setData( data, 0, 0d );
					colorBar( 1 );
					lastDirection = direction;
				}
				if (CurrentBar < 1)
				{
					risingColor = this.Plots[0].Pen.Color;
					fallingColor = this.Plots[1].Pen.Color;
					setData( data, 0, 0.0d);
					return;
				}
				colorBar( 0 );
			}
			catch( Exception ex )
			{
				//Print( ex.ToString() );
			}
			
			if (Signal[0] == 1)
				dmv.Set(RisingPlot[0]);
			if (Signal[0] == -1)
				dmv.Set(FallingPlot[0]);
			if (Signal[0] == 0)
				dmv.Set(NeutralPlot[0]);
				
			
        }


		private void colorBar( int bar )
		{
			try
			{
				direction = 0;
				double val1Pure = MAV( MAV(Input, MA1Period, (int)MA1Type ), MA2Period, (int)MA2Type)[ bar ];
				double val2Pure = MAV( MAV(Input, MA1Period, (int)MA1Type ), MA2Period, (int)MA2Type)[ bar + 1 ];
				double val1 = Math.Round( val1Pure, decimals );
				double val2 = Math.Round( val2Pure, decimals );
				
			
				bool rising = false;
				bool falling = false;
				if( val1 > val2 )
					rising = true;
				if( val1 < val2 )
					falling = true;
				
				if( Rising( MAV( MAV(Input, MA1Period, (int)MA1Type ), MA2Period, (int)MA2Type) ) )
				{ direction = 1; Signal.Set(1); }
				else if( Falling( MAV( MAV(Input, MA1Period, (int)MA1Type ), MA2Period, (int)MA2Type) ) )
				{ direction -= 1; Signal.Set(-1); }
				else
				{ direction = 0; Signal.Set(0); }

				
				if(rising)
				{
					RisingPlot.Set(bar + 1, val2);
					RisingPlot.Set(bar, val1);
					if (PaintBar)
						BarColor = barColorUp;
					Signal.Set(1);
				}
				else if(falling)
				{
					FallingPlot.Set(bar + 1, val2);
					FallingPlot.Set(bar, val1);
					if (PaintBar)
						BarColor = barColorDown;
					Signal.Set(-1);
				}
				else
				{
					NeutralPlot.Set(bar + 1, val2);
					NeutralPlot.Set(bar, val1);
					if (PaintBar)
						BarColor = barColorNeutral;					
					Signal.Set(0);
				}

				if( ShowArrows && CurrentBar > 2)
				{
					//if( lastDirection < direction )
					if (Signal[0] == 1 && Signal[1] != 1)
					{
						double val = Low[bar+1] - ( TickSize * ArrowDisplacement );
						DrawArrowUp(CurrentBar.ToString(),1, val , barColorUp );
						//data[ data.Count - 1] = val  * -1;
						//setData( data, bar, val  * -1 );
						if (soundon && FirstTickOfBar){						
							PlaySound(longwavfilename);
						}		
					}
					//else if( lastDirection > direction )
					else if (Signal[0] == -1 && Signal[1] != -1)
					{
						double val = High[bar+1] + ( TickSize * ArrowDisplacement );
						DrawArrowDown(CurrentBar.ToString(),1, val, barColorDown );
						if (soundon && FirstTickOfBar){						
							PlaySound(shortwavfilename);
						}
						//setData( data, bar, val );						
					}	
				
					
				
				}
			}
			catch( Exception ex )
			{
				//Print( ex.ToString() );
			}
		}

		private void setData( ArrayList list, int index, double val )
		{
			if( list == null || index < 0 )
				return;
			if( list.Count - 1 >= CurrentBar )
				list[ list.Count - index - 1 ] = val;
			else if( index == 0 )
				list.Add( val );
		}

		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			// Default plotting in base class. 
			base.Plot(graphics, bounds, min, max);
			
			if (base.Bars == null || !ShowArrows) return;
			
        	int index = -1;
			Exception caughtException;
				
            int barPaintWidth = base.ChartControl.ChartStyle.GetBarPaintWidth(base.ChartControl.BarWidth);
			int triangleWidth = 10;
			int arrowHeight = 15;
			int barWidth = 4;
			int bars = base.ChartControl.BarsPainted;
            while (bars >= 0)
            {
				index = ((base.ChartControl.LastBarPainted - base.ChartControl.BarsPainted) + 1) + bars;
                if (base.ChartControl.ShowBarsRequired || ((index - base.Displacement) >= base.BarsRequired))
                {
					try
					{
						if( data.Count > index && (double)data[ index ] != 0 )
						{
							double val = (double)data[ index ];
							int dir = 1;
							if( val < 0 )
							{
								val = val * -1;
								dir = -1;
							}
							
							Brush brush;
							int x1 = (((base.ChartControl.CanvasRight - base.ChartControl.BarMarginLeft - barPaintWidth ) ) - ((base.ChartControl.BarsPainted - 1) * base.ChartControl.BarSpace)) + (bars * base.ChartControl.BarSpace);
							int y1 = (bounds.Y + bounds.Height) - ((int) (((val - min) / ChartControl.MaxMinusMin(max, min)) * bounds.Height));
							int triangleTip;
							//int y2 = (bounds.Y + bounds.Height) - ((int) (((Values[2].Get(index) - min) / ChartControl.MaxMinusMin(max, min)) * bounds.Height));
							if( barPaintWidth > triangleWidth )
							{
								x1 += ( barPaintWidth - triangleWidth ) / 2;
							}
							else if( barPaintWidth < triangleWidth )
							{
								x1 -= ( triangleWidth - barPaintWidth ) / 2;
							}
							y1 = y1 + (int)( 0.5 * ( ( dir * -1 ) * triangleWidth ) );

							if( dir > 0 )
							{
								brush = new SolidBrush( fallingColor );
								triangleTip = y1 + triangleWidth / 2;
							}
							else
							{
								triangleTip = y1 - triangleWidth / 2;
								brush = new SolidBrush( risingColor );
							}
							Point[] points = new Point[]{
								      new Point( x1, y1)
									, new Point( x1 + ( ( triangleWidth - barWidth ) / 2 ), y1 )
									, new Point( x1 + ( ( triangleWidth - barWidth ) / 2 ), y1 + ( dir * -1 * arrowHeight ) )
									, new Point( x1 + barWidth + ( ( triangleWidth - barWidth ) / 2 ), y1 + ( dir * -1 * arrowHeight ) )
									, new Point( x1 + barWidth + ( ( triangleWidth - barWidth ) / 2 ), y1 )
									, new Point( x1 + triangleWidth, y1 )
									, new Point( x1 + ( triangleWidth / 2 ), triangleTip ) };
							graphics.FillPolygon( brush, points );
							Pen pen = new Pen( Color.FromKnownColor( KnownColor.Black ) );
							graphics.DrawPolygon( pen, points );

						}
					}
					catch (Exception exception)
						{
							caughtException = exception; 
							//Print( exception.ToString() ); 
						}
					
               	}
				bars--;					
			}
		}

		#region Properties		
       	[Description(" Paint Bar")]
        [Category("General")]
        public bool PaintBar
        {
            get { return paintBar; }
            set { paintBar = value; }
        }		
		[Description(" Sound")]
        [Category("General")]
        public bool SoundOn
        {
            get { return soundon; }
            set { soundon = value; }
        }		
		
		[Browsable(true)]
       	[XmlIgnore()]
        [Description("Color of up bars.")]
        [Category("Plots")]
        [Gui.Design.DisplayNameAttribute("Color Up")]
        public Color BarColorUp
        {
            get { return barColorUp; }
            set { barColorUp = value; }
        }
		[Browsable(true)]
       	[XmlIgnore()]
        [Description("Color of Neutral bars.")]
        [Category("Plots")]
        [Gui.Design.DisplayNameAttribute("Color Neutral")]
        public Color BarColorNeutral
        {
            get { return barColorNeutral; }
            set { barColorNeutral = value; }
        }
		[Browsable(true)]
       	[XmlIgnore()]
        [Description("Color of Down bars.")]
        [Category("Plots")]
        [Gui.Design.DisplayNameAttribute("Color Down")]
        public Color BarColorDown
        {
            get { return barColorDown; }
            set { barColorDown = value; }
        }
		
        [Browsable(false)]
        public string barColorUpSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorUp); }
            set { barColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Browsable(false)]
        public string barColorNeutralSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorNeutral); }
            set { barColorNeutral = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Browsable(false)]
        public string barColorDownSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorDown); }
            set { barColorDown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Signal
        {
            get { return signal; }
        }
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DMV
        {
            get { return dmv; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries RisingPlot
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries FallingPlot
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
		
        public DataSeries NeutralPlot
        {
            get { return Values[2]; }
        }

        [Description("")]
        [Category("Parameters")]
		
        public int MA2Period
        {
            get { return mA2Period; }
            set { mA2Period = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int MA1Period
        {
            get { return mA1Period; }
            set { mA1Period = Math.Max(1, value); }

        }

		[Description("Moving Average Type")]
        [Category("Parameters")]
		public NinjaTrader.Indicator.MAV.MAType MA1Type
		{
			get { return mA1Type; }
            set { mA1Type = value; }
		}

		[Description("Moving Average Type")]
        [Category("Parameters")]
		public NinjaTrader.Indicator.MAV.MAType MA2Type
		{
			get { return mA2Type; }
            set { mA2Type = value; }
		}
		
        [Description("Show Arrows")]
        [Category("General")]
        public bool ShowArrows
        {
            get { return showArrows; }
            set { showArrows = value; }
        }

        [Description("Draw Arrows above/below bars")]
        [Category("Plots")]
        public int ArrowDisplacement
        {
            get { return arrowDisplacement; }
            set { arrowDisplacement = Math.Max( 0, value ); }
        }
		
		[Description("Sound file to play for long alert.")]
		[Category("General")]
		public string WavLongFileName
		{
			get { return longwavfilename; }
			set { longwavfilename = value; }
		}
		[Description("Sound file to play for short alert.")]
		[Category("General")]
		public string WavShortFileName
		{
			get { return shortwavfilename; }
			set { shortwavfilename = value; }
		}		
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private DoubleMAPaint[] cacheDoubleMAPaint = null;

        private static DoubleMAPaint checkDoubleMAPaint = new DoubleMAPaint();

        /// <summary>
        /// DoubleMAPaint
        /// </summary>
        /// <returns></returns>
        public DoubleMAPaint DoubleMAPaint(int mA1Period, NinjaTrader.Indicator.MAV.MAType mA1Type, int mA2Period, NinjaTrader.Indicator.MAV.MAType mA2Type)
        {
            return DoubleMAPaint(Input, mA1Period, mA1Type, mA2Period, mA2Type);
        }

        /// <summary>
        /// DoubleMAPaint
        /// </summary>
        /// <returns></returns>
        public DoubleMAPaint DoubleMAPaint(Data.IDataSeries input, int mA1Period, NinjaTrader.Indicator.MAV.MAType mA1Type, int mA2Period, NinjaTrader.Indicator.MAV.MAType mA2Type)
        {
            checkDoubleMAPaint.MA1Period = mA1Period;
            mA1Period = checkDoubleMAPaint.MA1Period;
            checkDoubleMAPaint.MA1Type = mA1Type;
            mA1Type = checkDoubleMAPaint.MA1Type;
            checkDoubleMAPaint.MA2Period = mA2Period;
            mA2Period = checkDoubleMAPaint.MA2Period;
            checkDoubleMAPaint.MA2Type = mA2Type;
            mA2Type = checkDoubleMAPaint.MA2Type;

            if (cacheDoubleMAPaint != null)
                for (int idx = 0; idx < cacheDoubleMAPaint.Length; idx++)
                    if (cacheDoubleMAPaint[idx].MA1Period == mA1Period && cacheDoubleMAPaint[idx].MA1Type == mA1Type && cacheDoubleMAPaint[idx].MA2Period == mA2Period && cacheDoubleMAPaint[idx].MA2Type == mA2Type && cacheDoubleMAPaint[idx].EqualsInput(input))
                        return cacheDoubleMAPaint[idx];

            DoubleMAPaint indicator = new DoubleMAPaint();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.MA1Period = mA1Period;
            indicator.MA1Type = mA1Type;
            indicator.MA2Period = mA2Period;
            indicator.MA2Type = mA2Type;
            indicator.SetUp();

            DoubleMAPaint[] tmp = new DoubleMAPaint[cacheDoubleMAPaint == null ? 1 : cacheDoubleMAPaint.Length + 1];
            if (cacheDoubleMAPaint != null)
                cacheDoubleMAPaint.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheDoubleMAPaint = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// DoubleMAPaint
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DoubleMAPaint DoubleMAPaint(int mA1Period, NinjaTrader.Indicator.MAV.MAType mA1Type, int mA2Period, NinjaTrader.Indicator.MAV.MAType mA2Type)
        {
            return _indicator.DoubleMAPaint(Input, mA1Period, mA1Type, mA2Period, mA2Type);
        }

        /// <summary>
        /// DoubleMAPaint
        /// </summary>
        /// <returns></returns>
        public Indicator.DoubleMAPaint DoubleMAPaint(Data.IDataSeries input, int mA1Period, NinjaTrader.Indicator.MAV.MAType mA1Type, int mA2Period, NinjaTrader.Indicator.MAV.MAType mA2Type)
        {
            return _indicator.DoubleMAPaint(input, mA1Period, mA1Type, mA2Period, mA2Type);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// DoubleMAPaint
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DoubleMAPaint DoubleMAPaint(int mA1Period, NinjaTrader.Indicator.MAV.MAType mA1Type, int mA2Period, NinjaTrader.Indicator.MAV.MAType mA2Type)
        {
            return _indicator.DoubleMAPaint(Input, mA1Period, mA1Type, mA2Period, mA2Type);
        }

        /// <summary>
        /// DoubleMAPaint
        /// </summary>
        /// <returns></returns>
        public Indicator.DoubleMAPaint DoubleMAPaint(Data.IDataSeries input, int mA1Period, NinjaTrader.Indicator.MAV.MAType mA1Type, int mA2Period, NinjaTrader.Indicator.MAV.MAType mA2Type)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.DoubleMAPaint(input, mA1Period, mA1Type, mA2Period, mA2Type);
        }

    }
}
#endregion
